@@ -4,7 +4,22 @@
 */
 /* #define _POSIX_C_SOURCE 199309L whatever that may mean...*/ 
 /* #define _BSD_SOURCE             I use strdup, S_IFDIR, etc */ 
-/* #define _GNU_SOURCE */
+
+/* Roderich Schupp writes (bug #79100):
+   /usr/include/dlfcn.h from libc6 2.2-5 defines RTLD_NEXT only
+   when compiled with _GNU_SOURCE defined. Hence libfakeroot.c doesn't pick
+   it
+   up and does a dlopen("/lib/libc.so.6",...) in get_libc().
+   This works most of the time, but explodes if you have an arch-optimized
+   libc installed: the program now has two versions of libc.so
+   (/lib/libc.so.6 and, say, /lib/i586/libc.so.6) mapped. Again for
+   some programs you might get away with this, but running bash under
+   fakeroot
+   always bombs. Simple fix:
+*/
+#define _GNU_SOURCE 
+
+
 
 #include "communicate.h"
 #include <stdlib.h>
@@ -58,6 +73,12 @@
 #undef __fxstat64
 #undef __lxstat64
 
+/*
+// next_wrap_st:
+// this structure is used in next_wrap, which is defined in
+// wrapstruct.h, included below
+*/
+ 
 struct next_wrap_st{
   void **doit;
   char *name;
@@ -85,7 +106,7 @@ void load_library_symbols(void);
 
 void load_library_symbols(void){
   /* this function loads all original functions from the C library.
-     I ran into problems when let each function individually
+     I ran into problems when  each function individually
      loaded it's original counterpart, as RTLD_NEXT seems to have
      a different meaning in files with different names than libtricks.c
      (I.E, dlsym(RTLD_NEXT, ...) called in vsearch.c returned funtions
@@ -484,6 +505,9 @@ int WRAP_MKNOD MKNOD_ARG(int ver UNUSED,
 int mkdir(const char *path, mode_t mode){
   struct stat st;
   int r;
+  mode_t old_mask=umask(022);
+
+  umask(old_mask);
 
 
   /* we need to tell the fake deamon the real mode. In order
@@ -499,7 +523,8 @@ int mkdir(const char *path, mode_t mode){
   if(r)
     return -1;
   
-  st.st_mode=(mode&ALLPERMS)|(st.st_mode&~ALLPERMS)|S_IFDIR;
+  st.st_mode=(mode&~old_mask&ALLPERMS)|(st.st_mode&~ALLPERMS)|S_IFDIR;
+
   send_stat(&st, chmod_func);
 
   return 0;
@@ -622,3 +647,15 @@ uid_t getgid(void){
 uid_t getegid(void){
   return faked_egid();
 }
+
+int setuid(uid_t id){
+  return 0;
+}
+
+int setgid(uid_t id){
+  return 0;
+}
+
+int initgroups(const char* user, gid_t group){
+  return 0;
+}
